package com.perforce.ts.rest;

import com.atlassian.jira.rest.client.api.JiraRestClient;
import com.atlassian.jira.rest.client.api.RestClientException;
import com.atlassian.jira.rest.client.api.domain.Issue;
import com.atlassian.jira.rest.client.api.domain.ServerInfo;
import com.atlassian.jira.rest.client.api.domain.input.FieldInput;
import com.atlassian.jira.rest.client.api.domain.input.IssueInput;
import com.atlassian.jira.rest.client.api.domain.util.ErrorCollection;
import com.atlassian.jira.rest.client.internal.async.AsynchronousJiraRestClientFactory;
import com.atlassian.util.concurrent.Promise;
import java.net.URI;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import org.joda.time.DateTime;

/**
 * For testing connections:
 * connects to a jira server using REST, basic auth
 * (optional) retrieves an issue
 * (options) updates the issue's summary
 * .
 *
 * @author jbrown
 */
public class RestConnect {

    private final AsynchronousJiraRestClientFactory factory = new AsynchronousJiraRestClientFactory();

    /**
     * connect to JIRA and (optionally) retrieve an issue
     *
     * @param url
     * @param user
     * @param password
     * @param issueName optional. Retrieve if found.
     * @param updateSummary optional. Append this String to the summary and update it.
     * @return 0 for success, 1 for issue not found
     * @throws Exception
     */
    public int connectAndRun(String url, String user, String password, String issueName, String updateSummary) throws Exception {

        URI jiraServerUri = new URI(url);
        JiraRestClient restClient = factory.createWithBasicHttpAuthentication(jiraServerUri, user, password);
        System.out.println("Connected to " + jiraServerUri);
        
        restClientManager = new RestClientManager(url, user, password);
        try {
            String serverDate = getServerDate();
            System.out.println("Server Date: " + serverDate);
        } catch (Exception exception) {
            System.err.println("Exception on getting server date: " + exception.getMessage());
            exception.printStackTrace();
        }
        
        if (issueName != null && issueName.length() > 2) {
            try {


                Issue issue = getIssue(restClient, issueName);

                if (issue == null) {
                    System.out.println("Issue '" + issueName + "' not found.");
                    return 2;
                }
                String summary = issue.getSummary();
                System.out.println("Issue '" + issueName + "'  found: " + summary);

                if (updateSummary != null) {
                    summary = summary + updateSummary;
                    List<FieldInput> fields = new ArrayList();
                    fields.add(new FieldInput("summary", summary));
                    updateIssue(issue, url, user, password, fields);

                    issue = getIssue(restClient, issueName);
                    if (issue != null) {
                        System.out.println("Issue '" + issueName + "'  updated, summary: " + issue.getSummary());
                    }
                }
            } finally {
                restClient.close();
            }
        }
        return 0;
    }

    /**
     * Returns an issue. Returns null if issue not found. Other problems throw an
     * RestClientException
     *
     * @param restClient
     * @param issueName
     * @return
     */
    private Issue getIssue(JiraRestClient restClient, String issueName) throws RestClientException {
        Issue issue = null;
        try {
            issue = restClient.getIssueClient().getIssue(issueName).claim();
        } catch (RestClientException e) {

            Collection<ErrorCollection> errs = e.getErrorCollections();
            for (ErrorCollection err : errs) {
                if (err.getStatus() == 404) {
                    return (Issue)null;
                }
            }
            throw e;
        }
        return issue;
    }

    private void updateIssue(Issue issue, String url, String user, String password, List<FieldInput> fieldInputs) throws Exception {
        if (restClientManager == null) {
            throw new Exception("Error occurred while connecting to the JIRA server: "
                + "rest client manager initialization error.");
        }
        // Create issue input with specified fields
        IssueInput issueInput = IssueInput.createWithFields(fieldInputs.toArray(
            new FieldInput[fieldInputs.size()]));
        // Update the other fields on the issue
        restClientManager.getExtendedIssueClient().update(issue, issueInput).claim();
    }
    RestClientManager restClientManager = null; // new RestClientManager(url, user, password);

    public String getServerDate() throws Exception {
        DateFormat format = new SimpleDateFormat(DATE_PATTERN);
        Date serverDate = null;
        // ServerInfo serverInfo = restClientManager.getExtendedMetadataClient().getServerInfo().claim();
        ExtendedMetadataRestClient  mdClient =  restClientManager.getExtendedMetadataClient() ;
        Promise<ServerInfo> si = mdClient.getServerInfo();
        ServerInfo serverInfo = si.claim();
        if (serverInfo != null) {
            DateTime serverTime = serverInfo.getServerTime();
            serverDate = serverTime.toDate();
        } else {
            throw new Exception("serverInfo is null!");
        }
        return format.format(serverDate);
    }
    public static final String DATE_PATTERN = "yyyy/MM/dd HH:mm:ss";
}
